#!/usr/bin/python

from z3 import *
import argparse
import itertools
import time
from subprocess import call

n = 500
d = 3
limit = 4000
start = 3000
delta = 20

# reading edges from a file
# the file has 7000 edges for graph with 500 nodes
edges = []
with open("edges.txt") as f:
    for line in f:
        edges.append([int(v) for v in line.strip().split(',')])

# declare variables d color problem n 500 nodes
vs = [ [ Bool ("e_{}_{}".format(i,j)) for j in range(d)] for i in range(n)]

def at_least_one( ls ):
    return Or( ls )

Node_Fs = []
# each nodes has at least one color
for i in range(n):
    ls = []
    for j in range(d):
        ls.append(vs[i][j])
    Node_Fs.append( at_least_one(ls) )

# neigbouring nodes do not have same color
count = 0
edge_Fs = []
for e in edges:
    i1 = e[0]
    i2 = e[1]
    for j in range(d):
        edge_Fs.append( Or( Not( vs[i1][j] ), Not( vs[i2][j] ) ) )
    if count > limit:
        break
    count = count + 1

edge_num = start
data = open("graph-color.data", "w")
while edge_num < limit:
    st = time.time()
    s = Solver()
    s.add( And(Node_Fs) )
    s.add( And(edge_Fs[0:edge_num]) ) # consider only first edge_num edges
    r = s.check()
    tt = time.time() - st
    data.write( str(edge_num) + " " + str(tt) + " " )
    print edge_num,
    print tt,
    if r == sat:
        data.write( "1\n" )
        print "sat"
    else:
        data.write( "0\n" )
        print "unsat"
    edge_num = edge_num + delta
data.close()

call(["gnuplot", "-persist", "graph-color-plot.p"])
